/*
 * jQuery pretty date plug-in 1.0.0
 * 
 * http://bassistance.de/jquery-plugins/jquery-plugin-prettydate/
 * 
 * Based on John Resig's prettyDate http://ejohn.org/blog/javascript-pretty-date
 *
 * Copyright (c) 2009 Jörn Zaefferer
 *
 * $Id: jquery.validate.js 6096 2009-01-12 14:12:04Z joern.zaefferer $
 *
 * Dual licensed under the MIT and GPL licenses:
 *   http://www.opensource.org/licenses/mit-license.php
 *   http://www.gnu.org/licenses/gpl.html
 */

(function() {

jQuery.prettyDate = {
	
	template: function(source, params) {
		if ( arguments.length == 1 ) 
			return function() {
				var args = jQuery.makeArray(arguments);
				args.unshift(source);
				return jQuery.prettyDate.template.apply( this, args );
			};
		if ( arguments.length > 2 && params.constructor != Array  ) {
			params = jQuery.makeArray(arguments).slice(1);
		}
		if ( params.constructor != Array ) {
			params = [ params ];
		}
		jQuery.each(params, function(i, n) {
			source = source.replace(new RegExp("\\{" + i + "\\}", "g"), n);
		});
		return source;
	},
	
	now: function() {
		//return new Date();
	    d = new Date();
	    utc = d.getTime() + (d.getTimezoneOffset() * 60000);
	    nd = new Date(utc);
	    return nd;
	},
	
	/*
    d = new Date();
    utc = d.getTime() + (d.getTimezoneOffset() * 60000);
    nd = new Date(utc);
	*/
	
	// Takes an ISO time and returns a string representing how
	// long ago the date represents.
	format: function(time) {
		var date = new Date((time || "").replace(/-/g,"/").replace(/[TZ]/g," ")),
			diff = (jQuery.prettyDate.now().getTime() - date.getTime()) / 1000,
			day_diff = Math.floor(diff / 86400);
			
		if ( isNaN(day_diff) || day_diff < 0 || day_diff >= 31 )
			return;
		
		var messages = jQuery.prettyDate.messages;
		return day_diff == 0 && (
				diff < 60 && messages.now ||
				diff < 120 && messages.minute ||
				diff < 3600 && messages.minutes(Math.floor( diff / 60 )) ||
				diff < 7200 && messages.hour ||
				diff < 86400 && messages.hours(Math.floor( diff / 3600 ))) ||
			day_diff == 1 && messages.yesterday ||
			day_diff < 7 && messages.days(day_diff) ||
			day_diff < 31 && messages.weeks(Math.ceil( day_diff / 7 ));
	}
	
};

jQuery.prettyDate.messages = {
	now: "just now",
	minute: "1 minute ago",
	minutes: jQuery.prettyDate.template("{0} minutes ago"),
	hour: "1 hour ago",
	hours: jQuery.prettyDate.template("{0} hours ago"),
	yesterday: "Yesterday",
	days: jQuery.prettyDate.template("{0} days ago"),
	weeks: jQuery.prettyDate.template("{0} weeks ago")
};

jQuery.fn.prettyDate = function(options) {
	options = jQuery.extend({
		value: function() {
			return jQuery(this).attr("title");
		},
		interval: 100000
	}, options);
	var elements = this;
	function format() {
		elements.each(function() {
			var date = jQuery.prettyDate.format(options.value.apply(this));
			//alert(date);
			if ( date && jQuery(this).text() != date )
				jQuery(this).text( date );
		});
	}
	format();
	if (options.interval)
		setInterval(format, options.interval);
	return this;
};

})();